SystemOrganization addCategory: #BBCRadioRecorder!

Object subclass: #BBCOutlet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCOutlet commentStamp: 'jrm 3/1/2014 12:52' prior: 0!
A BBCOutlet is required to manage a variety of address patterns used by different BBC outlets. Radio 1 and Radio 4 seem to be the problem children.

http://www.bbc.co.uk/radio1/programmes/schedules/england/today.xml
http://www.bbc.co.uk/1xtra/programmes/schedules/today.xml
http://www.bbc.co.uk/radio2/programmes/schedules/today.xml
http://www.bbc.co.uk/radio3/programmes/schedules/today.xml
http://www.bbc.co.uk/radio4/programmes/schedules/lw/today.xml
http://www.bbc.co.uk/radio4/programmes/schedules/fm/today.xml
http://www.bbc.co.uk/radio4extra/programmes/schedules/today.xml
http://www.bbc.co.uk/5live/programmes/schedules/today
http://www.bbc.co.uk/5livesportsextra/programmes/schedules.xml
http://www.bbc.co.uk/6music/programmes/schedules.xml
http://www.bbc.co.uk/worldserviceradio/programmes/schedules/today.xml

hmmm... new information

http://www.bbc.co.uk/radio4extra/programmes/schedules/this_week.xml
http://www.bbc.co.uk/radio4extra/programmes/schedules/last_week.xml


Instance Variables
!

Object subclass: #BBCRadioAvailabilitySchedule
	instanceVariableNames: 'outlet startDtime endDtime broadcasts'
	classVariableNames: 'OutletUrlStub'
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRadioAvailabilitySchedule commentStamp: 'jrm 3/1/2014 12:38' prior: 0!
A BBCRadioAvailabilitySchedule is the collection of broadcasts which are currently (at the time the object in instantiated) available for listening. The BBC Calls this Listen Again. Most/all programs are supposed to be available for seven days following the datetime of broadcast. Since a specific program can be re-broadcast within a few days or hours of each other, we must chose the latest broadcast date in the collection to create a single instance for that show. BBC provides weekly schedules by outlet.

Class Variables

OutletUrlStub
	- A Dictionary mapping outlet names to the schedule url stub for the outlet. We do this because there is a url for 'this_week' and 'last_week' with a variety of stubs.

Instance Variables

broadcasts
	- An OrderedCollection of broadcast objects which contain all of the information required to generate a command line string to be executed by the OS. (Sqeak cannot currently record audio streams.

endDtime
	- The datetime, in Z, of the end of the schdule coverage period. Determined by the latest availability date for any broadcast in the collection
	
startDtime
	- The datetime, in Z, of the start of the schdule coverage period. Determined by the earliest availability date for any broadcast in the collection
	
outlet
	- The BBC Radio station identifier for the schedule.

outlet, start and endDtime are derived from the content of broadcasts.
!

!BBCRadioAvailabilitySchedule class methodsFor: 'class initialization' stamp: 'jrm 3/1/2014 13:09'!
initialize
	"BBCRadioAvailabilitySchedule initialize" 
	super initialize.
	OutletUrlStub := (Dictionary new).
	OutletUrlStub
		at: 'radio4extra' put: 'http://www.bbc.co.uk/radio4extra/programmes/schedules/';
		at: 'radio3' put: 'http://www.bbc.co.uk/radio3/programmes/schedules/';
		at: 'radio4' put: 'http://www.bbc.co.uk/radio4/programmes/schedules/fm/';
		at: 'radio2' put: 'http://www.bbc.co.uk/radio2/programmes/schedules/';
		at: '6music' put: 'http://www.bbc.co.uk/6music/programmes/schedules/';
		at: 'radio1' put: 'http://www.bbc.co.uk/radio1/programmes/schedules/england/';
		at: '1xtra' put: 'http://www.bbc.co.uk/1xtra/programmes/schedules/';
		yourself



	
! !

!BBCRadioAvailabilitySchedule class methodsFor: 'as yet unclassified' stamp: 'jrm 3/1/2014 12:58'!
newFor: anOutlet
	"There is no one standard pattern for an outlet, so we build the appropriate urlStub here and create a new instance that will populate itself using the stub."

	"BBCRadioAvailabilitySchedule newFor: 'radio4extra' "

	|  newSchedule|
	newSchedule := self new.
	anOutlet = nil ifFalse:[
	^ newSchedule newFrom:  (OutletUrlStub at: anOutlet)]


	
! !

!BBCRadioAvailabilitySchedule class methodsFor: 'as yet unclassified' stamp: 'jrm 3/1/2014 12:44'!
newFrom: aUrl
	"Build a schedule of all currently available programs. The tag, <is_available_mediaset_pc_sd>0</is_available_mediaset_pc_sd> indicates that the broadcasts is not available >1< means it is. Get data from 'this_week' and 'last_week'"

	"BBCRadioAvailabilitySchedule newFrom: 'http://www.bbc.co.uk/radio4extra/programmes/schedules/this_week.xml' "

	|  newSchedule |

	newSchedule := self new.
	
	^ newSchedule newFrom: aUrl


	
! !

!BBCRadioAvailabilitySchedule class methodsFor: 'as yet unclassified' stamp: 'jrm 3/1/2014 12:45'!
urlStubFor: anOutlet
	"There is no one standard pattern for an outlet, so we build the appropriate urlStub."

	"BBCRadioAvailabilitySchedule urlStubFor: 'radio4extra' "

	|  urlStub|

	^ urlStub := self argument: #('urlStubFor', anOutlet) asSymbol
	
	
	


	
! !

!BBCRadioAvailabilitySchedule methodsFor: 'as yet unclassified' stamp: 'jrm 2/16/2014 16:07'!
broadcasts

	
		^ broadcasts
	! !

!BBCRadioAvailabilitySchedule methodsFor: 'as yet unclassified' stamp: 'jrm 2/22/2014 22:06'!
broadcasts: anOrderedCollection

	
		^ broadcasts := anOrderedCollection
	! !

!BBCRadioAvailabilitySchedule methodsFor: 'as yet unclassified' stamp: 'jrm 2/18/2014 19:49'!
compressSchedule
	"Compress the schedule by removing multiple broadcast entries for a single title (shows
	are broadcast more than once)"
	
	| aDictionary temp anOrderedCollection |
	
	aDictionary := Dictionary new.
	temp := broadcasts collect: [:each | aDictionary at: each title put: each].
	anOrderedCollection := OrderedCollection new.
	aDictionary valuesDo: [ :each | anOrderedCollection add: each].
	broadcasts := anOrderedCollection.
	! !

!BBCRadioAvailabilitySchedule methodsFor: 'as yet unclassified' stamp: 'jrm 3/1/2014 12:44'!
newFrom: aUrlStub
	"Build a schedule of all currently available programs. The tag, <is_available_mediaset_pc_sd>0</is_available_mediaset_pc_sd> indicates that the broadcasts is not available >1< means it is. Get data from 'this_week' and 'last_week' using aUrlStub"

	"BBCRadioAvailabilitySchedule newFrom: 'http://www.bbc.co.uk/radio4extra/programmes/schedules/'' "

	|  xmlStream anXMLDocument root |

	broadcasts := OrderedCollection new.
	"Get 'this_week'"
	{ 'this_week' . 'last_week'} do: [:each | 
	
	xmlStream := HTTPClient httpGet: aUrlStub, each, '.xml'.
	anXMLDocument  := XMLDOMParser parseDocumentFrom: (xmlStream).
	root := anXMLDocument root.
	root tagsNamed: #'day' childrenDo: [:every | every tagsNamed: #'broadcast' do: [:one | 
			
	broadcasts add: (BBCRadioBroadcast newFrom: one) ]]].
	broadcasts removeAllSuchThat: [:each | each == nil].
	self compressSchedule.
	broadcasts sort: [:a :b | a title  > b title].


	
! !

Object subclass: #BBCRadioBroadcast
	instanceVariableNames: 'duration episodePid episodePosition episodeTitle episodeSynopsis seriesTitle seriesPosition seriesChildCount brandTitle brandPosition brandChildCount outlet availableStartTime availableEndTime availabilityComment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRadioBroadcast class methodsFor: 'as yet unclassified' stamp: 'jrm 2/18/2014 18:51'!
newFrom: anXMLElement
	"comment stating purpose of message"

	| broadcast |
	
	broadcast := self new.
	(anXMLElement firstTagNamed: #'available_until') == nil 
	ifFalse: [broadcast populateFrom: anXMLElement.
	^ broadcast]
		ifTrue: [^ nil]
			

! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/18/2014 20:06'!
availabilityLimit

	| aStream available |
	
	aStream := (WriteStream on: (String new: 16)).
	available := availableEndTime.
	 available == nil 
	ifFalse: [available printYMDOn: aStream] ifTrue: [aStream := '0000-00-00'].
	^aStream contents! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/8/2014 15:58'!
availableUntil

	"aDate"
	^ self programme availableUntil! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 5/5/2014 21:22'!
buildCommandLineString


	| anXMLDocument destinationFilePath episodeUrl mediaPid mediaUrl mediaPlaylist fileName mmshUrl cmdLine osp |
	destinationFilePath := '/Volumes/MediaVault/BBC/'.
	episodeUrl := 'http://www.bbc.co.uk/programmes/', episodePid, '.xml'.
	anXMLDocument  := XMLDOMParser
				parseDocumentFrom: (HTTPClient httpGet: episodeUrl).
	mediaPid := (( anXMLDocument root firstTagNamed: #version) firstTagNamed: #pid ) contentString.
fileName := self composeFileName.
" This all needs to be refactored -----------------------------------------"
"http://www.bbc.co.uk/mediaselector/4/mtis/stream/<canonicalPid> "

	mediaUrl := 'http://www.bbc.co.uk/mediaselector/4/asx/', mediaPid, '/iplayer_intl_stream_wma_lo_concrete'.
mediaPlaylist := (HTTPClient httpGet: mediaUrl).
(mediaPlaylist contents includesSubString: 'HTTP/1.1 404 Not Found') 
ifTrue: [mediaUrl :=
'http://open.live.bbc.co.uk/mediaselector/4/asx/', mediaPid,'/stream-nonuk-audio_streaming_wma_low_nonuk'.
	mediaPlaylist := (HTTPClient httpGet: mediaUrl). mediaPlaylist := (HTTPClient httpGet: mediaUrl)] .

	anXMLDocument := XMLDOMParser
				parseDocumentFrom: mediaPlaylist.

	mmshUrl :=(((anXMLDocument firstTagNamed: #Entry) firstTagNamed: #ref) attributeAt: 'href') copyReplaceAll: 'mms:' with: 'mmsh:'. 
"-----------------------------"	
	cmdLine := '/opt/local/bin/ffmpeg -i ', mmshUrl,  ' -y -map_metadata -1 -acodec libmp3lame -ab 128k ', destinationFilePath, fileName.
	
	"Requires OSProcess Package"

osp := ExternalUnixOSProcess command: cmdLine.
osp == nil ifTrue: [osp notify: 'Problem with ExternalUnixOSProcess. As yet unknown root cause'].
	! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 3/19/2014 22:53'!
composeFileName

	"build a reasonable string to be used as a filename in a command "
	| fileName option tempFileName |
	fileName := self title.
	tempFileName := ByteString new.
	fileName := fileName select: [ :each | each value ~= Character space].
	fileName := fileName select: [ :each | each value ~= (Character characterTable at: 64)].
	fileName do: [:each | each = $/ ifTrue: [each := '-'].
						   each = $: ifTrue: [each := '-'].
					         each = $' ifTrue: [each := '-' ]].
	"tempFileName := SequenceableCollection streamContents: [fileName]."
	fileName  replaceAll:  $/  with: $-. 
	fileName replaceAll:  $:  with: $-.
	fileName replaceAll:  $'  with: $-.
	fileName replaceAll:  $, with: $-.
	"Ask for the desired option"
	option := UserDialogBoxMorph confirm: 'File Name for Stream' withCRs title: 'Create new, or look up existing?' trueChoice: 'Lookup' falseChoice: 'Create'.  
	option ifTrue: [ fileName := self fileNameFromString: (self fileNameFromDirectory: fileName )] 
			ifFalse:[fileName := self fileNameFromString: fileName].	
	^ fileName, '.mp3'! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/3/2014 19:33'!
duration: aString


	duration := aString asInteger! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/7/2014 21:24'!
endTime


	^endTime! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/3/2014 19:09'!
endTime: aDate


	endTime := aDate! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/18/2014 22:01'!
episodePid

	^ episodePid
	
! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/16/2014 20:46'!
episodePid: aString

	episodePid := aString
	
! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/16/2014 20:48'!
episodePid: aBroadcastPid duration: aDuration startTime: aStart	endTime: anEnd programme: anXMLNode


	episodePid := aBroadcastPid.
	duration := aDuration asInteger.
	startTime :=aStart.
	endTime := anEnd.
	programme := self programme: anXMLNode! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/22/2014 22:12'!
fileNameFromDirectory: aString
	" Let me select a name from a list so I can keep the file names for a list similar. I will edit the file 	name that I select in a later step"
	"BBCRecorderApp fileNameFromList"
	
	| fileList fileNameLIst fileName|
	fileList := (FileList newOn: (FileDirectory on: '/Volumes/MediaVault/BBC')) entriesMatching: '*.mp3'.
	fileNameLIst := fileList collect: [ :each | each name].
	fileName := ListChooser chooseItemFrom: fileNameLIst title: 'Hint: ', aString.
	"Strip file type indicator string .mp3 from end of file"
	^ fileName == nil ifTrue: [^fileName] ifFalse: [^ fileName copyFrom: 1 to: (fileName size -4)]
! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/22/2014 21:50'!
fileNameFromString: aString
	"Modify the title string to create a useful file name to store the transcribed stream."
	
	
	^ FillInTheBlankMorph  request:  '.MP3 will be appended to file name' initialAnswer: aString centerAt: 500@500.! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/10/2014 21:39'!
mediaUrl


	^ programme mediaUrl! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/18/2014 23:00'!
populateFrom: anXMLNode

	| episodeProgram seriesProgram brandProgram |
	
	((anXMLNode firstTagNamed: #'available_until')  == nil) ifTrue: [ ^ nil ].
	
	duration := (anXMLNode firstTagNamed: #duration) contentString asInteger.
	availableStartTime := (anXMLNode firstTagNamed: #'actual_start') contentString asDateAndTime.
		availableEndTime := (anXMLNode firstTagNamed: #'available_until') contentString asDateAndTime.
		availabilityComment:= (anXMLNode firstTagNamed: #availability) contentString.
	
	episodeProgram := (anXMLNode firstTagNamed: #programme with: [:target | (target attributes at: 'type') = 'episode'] ).
	episodePid := (episodeProgram firstTagNamed: #pid) contentString.
	episodePosition := (episodeProgram firstTagNamed: #position) contentString.
	episodeTitle := (episodeProgram firstTagNamed: #title) contentString.
	 episodeSynopsis := (episodeProgram firstTagNamed: #'short_synopsis') contentString.
	
	seriesProgram := (anXMLNode firstTagNamed: #programme with: [:target | (target attributes at: 'type') = 'series'] ).
	seriesProgram == nil ifFalse: [
		seriesTitle := (seriesProgram firstTagNamed: #title) contentString.
		seriesPosition := (seriesProgram firstTagNamed: #position) contentString.
		seriesChildCount := (seriesProgram firstTagNamed: #'expected_child_count') contentString].		
		
	brandProgram := (anXMLNode firstTagNamed: #programme with: [:target | (target attributes at: 'type') = 'brand'] ).
	brandProgram == nil ifFalse: [
		brandTitle := (brandProgram firstTagNamed: #title) contentString.
		brandPosition := (brandProgram) contentString.
		brandChildCount := (brandProgram firstTagNamed: #'expected_child_count') contentString.	
		outlet := ((brandProgram firstTagNamed: #ownership) firstTagNamed: #service) attributeAt: 'key'].
	
	seriesTitle == nil ifTrue: [seriesTitle := ''].
	seriesPosition == nil ifTrue: [seriesPosition := ''].
	seriesChildCount == nil ifTrue: [seriesChildCount := ''].
	brandTitle == nil ifTrue: [brandTitle := ''].
	brandPosition == nil ifTrue: [brandPosition := ''].
	brandChildCount == nil ifTrue: [brandChildCount := ''].
	






! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/7/2014 22:23'!
programme

	^ programme! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 3/1/2014 12:42'!
programme: anXMLNode

	| programmeSeries programmeBrand available episodeProgram programme |
	programme := BBCRadioProgramme new.
	
	episodeProgram := (anXMLNode firstTagNamed: #programme with: [:target | (target attributes at: 'type') = 'episode'] ).
	episodePid := episodeProgram firstTagNamed: #pid.
	episodePosition := episodeProgram firstTagNamed: #position.
	episodeTitle := episodeProgram firstTagNamed: #title.
	 episodeSynopsis := episodeProgram firstTagNamed: #'short_synopsis'.
	
		"  This is a call to another web page. We do not need it now. Get it when it is needed
			mediaUrl: programmePid;"
			programme episodePid: episodePid; 

				 programmePosition: (anXMLNode firstTagNamed: #position) contentString;
				 programmeTitle: (anXMLNode firstTagNamed: #title) contentString;
				 synopsis: (anXMLNode firstTagNamed: #'short_synopsis') contentString.

				" #'available_until' is not present if the program is not currently available"
				available := anXMLNode firstTagNamed: #'available_until'.
				available   == nil   ifFalse: [programme availableUntil: (DateAndTime fromString:( available contentString)) ].
				 programmeSeries := (anXMLNode firstTagNamed: #programme with:[:each | (each valueFor: #type) = 'series']).
				 programmeBrand := (anXMLNode firstTagNamed: #programme with:[:each | (each valueFor: #type) = 'brand']).
				
				" have to deal with brands or series!! No consistancy in the XML files"  
	programmeSeries = nil ifFalse: [
	programme seriesTitle: (programmeSeries firstTagNamed: #title) contentString;
				 childCount: (programmeSeries firstTagNamed: #'expected_child_count') contentString;
				 seriesPosition: (programmeSeries firstTagNamed: #position) contentString].
	programmeBrand = nil ifFalse: [
	programme brandTitle: (programmeBrand firstTagNamed: #title)contentString].
self halt.
	^ programme! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/10/2014 21:33'!
programmePid

	^ programme programmePid! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/18/2014 21:53'!
scheduleMorph
	"create a single StringMorph for a single schedule entry."

	| tMorph t1 t2 t3 |	
		tMorph := StringButtonMorph new.
	t1 := (self title, '  ').
	t2 := self availabilityLimit.
	t3 := (episodeSynopsis).
	
	^tMorph contents: (t2, ' ', t1); setBalloonText: t3.! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/3/2014 19:09'!
startTime: aDate


	startTime := aDate! !

!BBCRadioBroadcast methodsFor: 'as yet unclassified' stamp: 'jrm 2/23/2014 15:18'!
title

	"seriesPosition   brandPosition brandChildCount "
	
	| displayNofM displayBrandTitle displaySeriesTitle displayEpisodeTitle |
	
	episodePosition = '' ifTrue: [ displayNofM := ''] ifFalse: [ displayNofM := ' ', episodePosition, '.', seriesChildCount, ' '].
	seriesPosition = '' ifFalse: [displayNofM := ' ', seriesPosition, '.', episodePosition, '.', seriesChildCount, ' '].
	( 'Episode*' match: episodeTitle ) ifTrue: [displayEpisodeTitle := ''] ifFalse: [displayEpisodeTitle := episodeTitle].
	('Series*'  match: seriesTitle) ifTrue: [displaySeriesTitle := ''] ifFalse: [displaySeriesTitle := seriesTitle].
	brandTitle = '' ifFalse: [displayBrandTitle := brandTitle, ' '] ifTrue: [displayBrandTitle := ''].
	
	^ displayBrandTitle,displaySeriesTitle, displayNofM, displayEpisodeTitle
	
	! !

Object subclass: #BBCRadioMediaSelector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRadioMediaSelector class methodsFor: 'as yet unclassified' stamp: 'jrm 2/7/2014 19:38'!
buildMediaSelectorString:aPid
	"A pid is a program identifier that can be used to create a uri which can be used to obtain an XML file which contains the mms address to a recordable stream. The iplayer uri is part of the stream from the mtis version. You can use it, but there is more extraneous data to work with. http://www.bbc.co.uk/mediaselector/4/mtis/stream/<pid> They both can be generated by inserting a pid. I am using the iplayer version, not the mtis one. http://www.bbc.co.uk/mediaselector/4/asx/b007qtjp/iplayer_intl_stream_wma_lo_concrete  "

	| episodeUrl episode anXMLDocument canonical mediaUrl |

	episodeUrl := 'http://www.bbc.co.uk/programmes/', aPid, '.xml'.

episode :=HTTPClient httpGet: episodeUrl.		
	anXMLDocument  := XMLDOMParser parseDocumentFrom: (episode).
	canonical := ((anXMLDocument firstTagNamed: #versions) firstTagNamed: #pid )contentString.
	^ mediaUrl := 'http://www.bbc.co.uk/mediaselector/4/asx/', canonical , '/iplayer_intl_stream_wma_lo_concrete' .
! !

!BBCRadioMediaSelector class methodsFor: 'as yet unclassified' stamp: 'jrm 3/1/2014 12:45'!
getHref: aPid
	" Href is a uri that   "

	| selector anXMLDocument root entry ref href |

	selector :=HTTPClient httpGet:   (BBCRadioMediaSelector buildMediaSelectorString: aPid).		
	anXMLDocument  := XMLDOMParser parseDocumentFrom: (selector).
	root := anXMLDocument root.
	entry := root firstTagNamed: #Entry.
	ref := entry firstTagNamed: #ref.
	href := ref attributeAt: #href.
	^ href := ref attributeAt: #href.
	
	! !

Object subclass: #BBCRadioProgramme
	instanceVariableNames: 'programmePid programmePosition programmeTitle synopsis childCount seriesTitle seriesPosition brandTitle mediaUrl availableUntil'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/9/2014 20:44'!
availabilityLimit

	| aStream available |
	
	aStream := (WriteStream on: (String new: 16)).
	available := self availableUntil.
	 available == nil 
	ifFalse: [available printYMDOn: aStream] ifTrue: [aStream := '0000-00-00'].
	^aStream contents! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/8/2014 15:40'!
availabilityLimit: aDate

	availabilityLimit := aDate! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/9/2014 20:21'!
availableUntil

	^ availableUntil! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/9/2014 20:24'!
availableUntil: aDate

	^ availableUntil := aDate! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/4/2014 19:12'!
brandTitle: aString

	brandTitle := aString.! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/3/2014 20:01'!
childCount: aString

	childCount := aString.! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/10/2014 21:37'!
mediaUrl


	^ mediaUrl := self mediaUrl: programmePid
! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 3/1/2014 12:41'!
mediaUrl: aProgrammePid


	^ mediaUrl := BBCRadioMediaSelector buildMediaSelectorString: aProgrammePid.
! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/10/2014 22:32'!
orderInSeries
	"aString - order/series - used in title"

	seriesPosition == nil ifTrue: [^''] ifFalse: [^ ' ' ,seriesPosition, '/', childCount]! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/10/2014 21:32'!
programmePid

	^ programmePid! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/3/2014 19:57'!
programmePid: aString

	programmePid := aString.! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/3/2014 20:07'!
programmePosition: aString

	programmePosition := aString.! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/3/2014 20:08'!
programmeTitle: aString

	programmeTitle := aString.! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/3/2014 20:02'!
seriesPosition: aString

	seriesPosition := aString.! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/3/2014 20:01'!
seriesTitle: aString

	seriesTitle := aString.! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/8/2014 15:13'!
synopsis

	^ synopsis! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/3/2014 20:01'!
synopsis: aString

	synopsis := aString.! !

!BBCRadioProgramme methodsFor: 'as yet unclassified' stamp: 'jrm 2/10/2014 22:32'!
title

"Brand or series title can be blank (or maybe just brand), but this will help"

	| bTitle sTitle |
	
	bTitle := brandTitle == nil ifTrue: [''] ifFalse: [brandTitle, ' '].
	sTitle := seriesTitle == nil ifTrue: [''] ifFalse: [seriesTitle, ' '].

	^ bTitle, sTitle,  programmeTitle, self orderInSeries! !

Object subclass: #BBCRadioSchedule
	instanceVariableNames: 'service day broadcasts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRadioSchedule commentStamp: 'jrm 3/1/2014 12:52' prior: 0!
A BBCRadioSchedule is an object to manage a broadcast schedule for shows that are available to "Listen Again"

Instance Variables
	broadcasts:		<Object>
	day:		-- deprecate
	service:		--- change to outlet ??

broadcasts
	- xxxxx

day
	- xxxxx

service
	- xxxxx
!

!BBCRadioSchedule class methodsFor: 'as yet unclassified' stamp: 'jrm 3/1/2014 12:54'!
newFrom: aUrlStub

"A complete schedule of progerams available to be recorded requires access to two schedules, this_week and last_week. "

	"BBCRadioSchedule newFrom: 'http://www.bbc.co.uk/radio4extra/programmes/schedules/' "
	

	| scheduleWeeks schedule anXMLDocument service broadcastsDoc broadcasts aSchedule |
	
	aSchedule := self new.
	broadcasts := OrderedCollection new.
	scheduleWeeks := {'this_week.xml' . 'last_week.xml'}.

	scheduleWeeks do: [:each |
	schedule := (HTTPClient httpGet: aUrlStub, each).		
	anXMLDocument  := XMLDOMParser parseDocumentFrom: (schedule).
	service := (anXMLDocument firstTagNamed: #service) attributes at: #key.

	broadcastsDoc :=anXMLDocument firstTagNamed: #broadcasts.
	broadcastsDoc elementsAndContents do: [:every | broadcasts add: (BBCRadioBroadcast newFrom: every) ]].
	
	^ aSchedule service: service  broadcasts: broadcasts.
	! !

!BBCRadioSchedule methodsFor: 'as yet unclassified' stamp: 'jrm 2/7/2014 20:07'!
broadcasts
	"comment stating purpose of message"


	^ broadcasts! !

!BBCRadioSchedule methodsFor: 'as yet unclassified' stamp: 'jrm 2/23/2014 12:34'!
initialize
	


	! !

!BBCRadioSchedule methodsFor: 'as yet unclassified' stamp: 'jrm 3/1/2014 12:53'!
service: aService broadcasts: aBroadcasts
	"comment stating purpose of message"

	service := aService.
	broadcasts := aBroadcasts.! !

!BBCRadioSchedule methodsFor: 'as yet unclassified' stamp: 'jrm 2/4/2014 19:43'!
service: aService day: aDay broadcasts: aBroadcasts
	"comment stating purpose of message"

	service := aService.
	day := aDay.
	broadcasts := aBroadcasts.! !

Object subclass: #BBCRecorderApp
	instanceVariableNames: 'availabilitySchedule aSystemWindow aPasteUpMorph osp2'
	classVariableNames: 'DefaultOutlet DestinationFolder'
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRecorderApp commentStamp: 'jrm 3/1/2014 12:47' prior: 0!
A BBCRecorderApp is An application to browse and record the Listen Again availability list for various BBC radio stations from outside the UK using the ffmpeg application.

Instance Variables
	aBBCRadioSchedule:		<Object>

availabilitySchedule
	- xxxxx
aBBCRadioWeeklySchedule
	- to be deprecated at some point
SystemWindow 
	- xxxxx
aPasteUpMorph 
	- xxxxx
osp2
	- is this needed as an instance var?


Class Variables

	DestinationFolder          Folder where recordings are written. User Selectable!

!BBCRecorderApp class methodsFor: 'initialize-release' stamp: 'jrm 2/7/2014 19:42'!
defaultDestination

"This will be used for insertion into a command string"
	^ DestinationFolder := '/Volumes/MediaVault/BBC/'! !

!BBCRecorderApp class methodsFor: 'initialize-release' stamp: 'jrm 2/7/2014 19:42'!
defaultOutlet

   	^ DefaultOutlet := 'radio4extra'! !

!BBCRecorderApp class methodsFor: 'initialize-release' stamp: 'jrm 3/1/2014 12:48'!
initialize

    "BBCRecorderApp initialize"

 	self defaultOutlet; defaultDestination; registerInWorldMenu.! !

!BBCRecorderApp class methodsFor: 'instance-creation' stamp: 'jrm 3/1/2014 12:48'!
openAsMorph
"BBCRecorderApp openAsMorph "

	^self new openAsMorph! !

!BBCRecorderApp class methodsFor: 'instance-creation' stamp: 'jrm 3/1/2014 12:50'!
registerInWorldMenu
	self environment at: #TheWorldMenu ifPresent: [ :class |
		class registerOpenCommand: (Array 
			with: 'BBC Recorder' 
			with: (Array
				with: self
				with: #openAsMorph)) ].! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
buttonSelected

self halt.

! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
buttonSelected: aBBCRadioScheduleEntry

	| linkCollection linkItem mediaSelectorUrl |

"When the button is selected, we want to compose and execute a command line string that will execute a program that will transcribe the program string into an MP3 file that can be played on an iPod. The information we need in in the aBBCRadioScheduleEntry"

linkCollection := aBBCRadioScheduleEntry links.
linkCollection size = 1 ifTrue: [ linkItem := linkCollection at: 1 ] ifFalse: [ self halt: 'I only expect one element in the collection. The requirements have changed and will require analysis and programming to correct.'].

mediaSelectorUrl := linkItem url.

BBCRadioMediaselectorBuilder newFrom: mediaSelectorUrl.

self halt.

! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 3/1/2014 12:49'!
buttonSelected: aStringButtonMorph entrySelected: aBBCRadioBroadcast


	"When the button is selected, we want to compose and execute a command line string that will execute a program that will transcribe the program string into an MP3 file that can be played on an iPod. The information we need in in the aBBCRadioBroadcast"

	" aStringButtonMorph is not needed here unless I want to highlight it while this work is being done "

	"self buildCommandLineStringFor: aStringButtonMorph with:  aBBCRadioBroadcast "

	^ aBBCRadioBroadcast buildCommandLineString! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
composeFileNameFor: aString seriesCount: sCountString episode: episodeString
	"We need a fileName to use in the command line when we record a stream. The file name is based on the title of the stream and we offer the option to edit the title directly or select a file name from a directory and edit it. The second option is most useful for a series when all you want to do is update the episode or series numbers"
	| option editString fullString |
	

	"Clean up problem characters"
	editString := aString replaceAll: $' with: $-.
	editString := editString replaceAll: $:  with: $-.
	editString  := editString replaceAll: $/  with: $-.
	fullString := editString,'.', episodeString,'.', sCountString.
	"Ask for the desired option"
	option := UserDialogBoxMorph confirm: 'File Name for Stream' withCRs title: 'Create new, or look up existing?' trueChoice: 'Lookup' falseChoice: 'Create'.  
	option ifTrue: [ ^ self fileNameFromString: (self fileNameFromDirectory: fullString )] 
			ifFalse:[^ self fileNameFromString: fullString]! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/23/2014 13:50'!
compressSchedule
	"Compress the schedule by removing multiple entries for a single title (shows
	are broadcast more than once"
	
	| aDictionary temp anOrderedCollection |
	
	aDictionary := Dictionary new.
	temp := availabilitySchedule broadcasts collect: [:each | aDictionary at: each title put: each].
	anOrderedCollection := OrderedCollection new.
	aDictionary valuesDo: [ :each | anOrderedCollection add: each].
	^ availabilitySchedule broadcasts: anOrderedCollection.
	! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 3/1/2014 12:48'!
currentSchedule
	""
	
	^ availabilitySchedule! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/12/2014 20:54'!
defaultOutlet
	
	
	^ 'radio4extra'! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
fileNameFromDirectory
	" Let me select a name from a list so I can keep the file names for a list similar. I will edit the file 	name that I select in a later step"
	"BBCRecorderApp fileNameFromList"
	
	| fileList fileNameLIst fileName|
	fileList := (FileList newOn: (FileDirectory on: '/Volumes/MediaVault/BBC')) entriesMatching: '*.mp3'.
	fileNameLIst := fileList collect: [ :each | each name].
	fileName := ListChooser chooseItemFrom: fileNameLIst.
	"Strip file type indicator string .mp3 from end of file"
	^ fileName == nil ifTrue: [^fileName] ifFalse: [^ fileName copyFrom: 1 to: (fileName size -4)]
! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
fileNameFromDirectory: aString
	" Let me select a name from a list so I can keep the file names for a list similar. I will edit the file 	name that I select in a later step"
	"BBCRecorderApp fileNameFromList"
	
	| fileList fileNameLIst fileName|
	fileList := (FileList newOn: (FileDirectory on: '/Volumes/MediaVault/BBC')) entriesMatching: '*.mp3'.
	fileNameLIst := fileList collect: [ :each | each name].
	fileName := ListChooser chooseItemFrom: fileNameLIst title: 'Hint: ', aString.
	"Strip file type indicator string .mp3 from end of file"
	^ fileName == nil ifTrue: [^fileName] ifFalse: [^ fileName copyFrom: 1 to: (fileName size -4)]
! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
fileNameFromString: aString seriesCount: sCountString episode: episodeString
	"Modify the title string to create a useful file name to store the transcribed stream."
	| aNewString fileName |
" (BBCRecorderApp new) fileNameFromString: 'Some thing new: series 5 3.5'"

	"Dump spaces from aString"
	aNewString := aString  select: [ :each | each value ~= Character space].
	episodeString = nil ifTrue:[ fileName := aNewString ] ifFalse: [fileName := aNewString,'.', episodeString,'.', sCountString].
      fileName :=	FillInTheBlankMorph  request:  '.raw will be appended to file name' initialAnswer: fileName centerAt: 500@500.	
	fileName := '"',fileName,  '.raw','"'.

self halt.	
	! !

!BBCRecorderApp methodsFor: 'initalization' stamp: 'jrm 2/7/2014 19:42'!
initialize
	"comment stating purpose of message"
	! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
menuFiles
	"Display the file list for review. User can copy string if desired???"
	
	
	
self fileNameFromDirectory.

! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 3/1/2014 13:12'!
menuOutlet
	"The BBC Radio Stations are called outlets. All we do here is get the new outlet and use it to set the aBBCRadioSchedule"
	

	
	
	| outlet |
	outlet := MenuMorph
		chooseFrom: #('radio4extra' 'radio4'  'radio3' 'radio2' '6music'  'radio1' '1xtra')
		values: #('radio4extra' 'radio4' 'radio3' 'radio2' '6music' 'radio1' '1xtra')
		lines: #()
		title: 'Outlet'.
	outlet = nil ifFalse: [
	 availabilitySchedule := (BBCRadioAvailabilitySchedule newFor: outlet).
	 aPasteUpMorph removeAllMorphs.
	self populateSchedule; compressSchedule.]
! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
menuSort
	"Display a menu of sort options Alpha Date. Perform the sort"
	
	
	| sortMethod |
	sortMethod := MenuMorph
		chooseFrom: #('Alpha' 'Date')
		values: #('sortAlpha' 'sortDate')
		lines: #()
		title: 'Sort by'.
	self perform: sortMethod asSymbol.
	

! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 3/1/2014 12:51'!
openAsMorph
	" eXperimenatl - refactoring to use BBCRadioAvailabilitySchedule.   create a BBC Radio Recorder Application on the BBC Radio4xtra schedule aScheduleMorph"
	
	"http://www.bbc.co.uk/radio1/programmes/schedules/england#on-now WTF"
	
	"BBCRecorderApp openAsMorph "
	| scrollPane submorphExtent menuButton1 menuButton2 mbHolder menuButton3 |
	
	availabilitySchedule := BBCRadioAvailabilitySchedule newFor: (BBCRecorderApp defaultOutlet).
	aSystemWindow := SystemWindow labelled: 'BBC'.
	aSystemWindow model: self.

mbHolder := (RectangleMorph new) layoutPolicy: TableLayout new;
	  listDirection: #leftToRight;
	 wrapCentering: #topLeft;
	  hResizing: #shrinkWrap;
	  vResizing: #shrinkWrap.
menuButton1 := SimpleButtonMorph new
		target: self ;
		label: 'Sort';
		actionSelector: #menuSort.
menuButton2 := (SimpleButtonMorph new
		target: self;
		label: 'Outlet';
		actionSelector: #menuOutlet;
		setBalloonText: 'Select a BBC Radio Outlet to display a list of available programs').
menuButton3 := (SimpleButtonMorph new
		target: self;
		label: 'FileNameReview';
		actionSelector: #menuFiles;
		setBalloonText: 'Take a look at the file names in the  current destination folder').
		
mbHolder addMorph: menuButton3; addMorph: menuButton1;addMorph: menuButton2.
aSystemWindow addMorph: mbHolder frame:  (0.0@0 corner: 1@0.007).
aPasteUpMorph := PasteUpMorph new.
aPasteUpMorph layoutPolicy: TableLayout new;
	  listDirection: #topToBottom;
	 wrapCentering: #topLeft;
	  hResizing: #shrinkWrap;
	  vResizing: #shrinkWrap.

self populateSchedule.

scrollPane := ScrollPane new.

scrollPane scroller addMorph: aPasteUpMorph.
aSystemWindow addMorph: scrollPane frame: (0@0.07 extent: 1@0.93).
submorphExtent := aPasteUpMorph submorphBounds corner.
scrollPane extent: submorphExtent.
aPasteUpMorph extent: submorphExtent; cellPositioning: #topLeft.
aPasteUpMorph beSticky. scrollPane beSticky.
aSystemWindow openInWorld.! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/22/2014 22:27'!
outletUrl: outlet
	"build the url using the value provided. If the value does not resolve correctl, there will be no file returned."

	outlet= 'radio4' ifTrue: [^ 'http://www.bbc.co.uk/radio4/programmes/schedules/fm/this_week.xml'].
	^ 'http://www.bbc.co.uk/', outlet, '/programmes/schedules/last_week.xml'! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/23/2014 10:54'!
outletUrlStub: outlet
	"Some of the BBC outlets have slightly different URLs. The final part of the complete string is this_week or last_week. Both URLs have to be accessed to get a complete list of programs available for recording."

	outlet= 'radio4' ifTrue: [^ 'http://www.bbc.co.uk/radio4/programmes/schedules/fm/'].

	^ 'http://www.bbc.co.uk/', outlet, '/programmes/schedules/'! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/23/2014 12:32'!
populateSchedule
	"Build aPasteUpMorph and add individual program schedule morphs into it.. Run once when a new schedule is selected into currentSchedule  "
	
	
| aScheduleMorph |
	availabilitySchedule broadcasts do: [:each   | aScheduleMorph := each scheduleMorph. 	aPasteUpMorph addMorph: (aScheduleMorph cellPositioning: #topLeft; target: self; actionSelector: #buttonSelected:entrySelected: ; arguments:  (Array with: aScheduleMorph with: each))].
! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/22/2014 22:05'!
sortAlpha
	"Sort the pick list in alpha order"
	
	aPasteUpMorph submorphsDo: [:each | each delete].
	availabilitySchedule broadcasts: (availabilitySchedule broadcasts sort: [:a :b | (a title, a availabilityLimit) > (b title, b availabilityLimit)]).

	self populateSchedule.! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/22/2014 22:17'!
sortDate
	"Sort the pick list in alpha order"
	
	aPasteUpMorph submorphsDo: [:each | each delete].
	availabilitySchedule broadcasts: (availabilitySchedule broadcasts sort: [:a :b | (a availabilityLimit) > (b availabilityLimit)]).

	self populateSchedule.
	
! !
BBCRadioAvailabilitySchedule initialize!
BBCRecorderApp initialize!
